
import { createRoot } from 'react-dom/client'
import App from './App.tsx'
import './index.css'
import { registerSW } from 'virtual:pwa-register'
import { preloadCriticalImages } from './utils/imageUtils';

// Register service worker with improved user feedback
const updateSW = registerSW({
  immediate: true, // Immediately register the service worker
  onNeedRefresh() {
    if (confirm('New content available. Reload for better performance?')) {
      updateSW(true)
    }
  },
  onOfflineReady() {
    console.log('App ready to work offline')
    // Show a toast if you want to notify users
  },
  onRegisterError(error) {
    console.error('SW registration error', error)
  }
})

// Preload critical images for faster LCP
preloadCriticalImages([
  '/calorie-calculator.jpg',
  '/calories-calculator.jpg',
  '/calorie-deficit-calculator.jpg'
]);

// Create root element with a reference we can clean up later
const rootElement = document.getElementById("root");

// Progressive hydration - serve a static shell first for faster FCP
if (rootElement) {
  // First contentful paint optimization - show something immediately
  rootElement.innerHTML = `
    <div id="initial-shell" style="min-height: 100vh; background-color: #f5f7f9; display: flex; flex-direction: column;">
      <div style="background-color: #ffffff; padding: 1rem; box-shadow: 0 1px 3px rgba(0,0,0,0.1);">
        <div style="max-width: 1200px; margin: 0 auto; display: flex; justify-content: space-between; align-items: center;">
          <div style="font-weight: bold; font-size: 1.25rem; color: #2e9063;">Calouri</div>
        </div>
      </div>
      <div style="text-align: center; padding-top: 2rem;">
        <p style="font-size: 1.25rem; color: #2e9063; font-weight: 500;">Loading Calorie Calculator...</p>
      </div>
    </div>
  `;

  // Add a performance mark to measure initial shell render
  performance.mark('shell-rendered');
}

// Create a function to hydrate the app
const renderApp = () => {
  if (!rootElement) return;

  // Add performance mark to measure hydration start
  performance.mark('hydration-start');
  
  const root = createRoot(rootElement);
  root.render(<App />);
  
  // Measure app hydration complete
  performance.mark('app-hydrated');
  performance.measure('app-hydration-time', 'hydration-start', 'app-hydrated');
  
  const hydrationPerf = performance.getEntriesByName('app-hydration-time')[0];
  console.log(`App hydration time: ${Math.round(hydrationPerf.duration)}ms`);
  
  // Clean up the shell after hydration to avoid duplicate content
  const initialShell = document.getElementById('initial-shell');
  if (initialShell) {
    initialShell.remove();
  }
};

// Use different hydration strategies based on environment and network
if (process.env.NODE_ENV === 'development') {
  // In development, render immediately for faster feedback
  renderApp();
} else {
  // In production, use optimized hydration strategy
  
  // Use requestIdleCallback for hydration to avoid blocking main thread
  if ('requestIdleCallback' in window) {
    window.requestIdleCallback(() => renderApp(), { timeout: 2000 });
  } else {
    // Fallback to setTimeout with a short delay for browsers without requestIdleCallback
    setTimeout(renderApp, 20);
  }
}

// In production, prefetch critical resources during idle time
if (process.env.NODE_ENV !== 'development') {
  // Use intersection observer to prefetch visible routes
  const observer = new IntersectionObserver((entries) => {
    entries.forEach(entry => {
      if (entry.isIntersecting && entry.target instanceof HTMLAnchorElement) {
        const href = entry.target.getAttribute('href');
        if (href && href.startsWith('/') && !href.includes('#')) {
          const link = document.createElement('link');
          link.rel = 'prefetch';
          link.href = href;
          document.head.appendChild(link);
          observer.unobserve(entry.target);
        }
      }
    });
  }, { threshold: 0.1 });

  // Start observing links after page load
  window.addEventListener('load', () => {
    document.querySelectorAll('a[href^="/"]').forEach(link => {
      observer.observe(link);
    });
  });
}
